/*
 * acct_dump.c
 * 
 * (!c) 2002 by Ighighi
 * Venezuela
 */

#include "config.h"

#include <stdio.h>
#include <time.h>
#include <pwd.h>
#include <grp.h>
#include <sys/acct.h>
#include <sys/types.h>		/* major() & minor() on BSD */
#ifndef major
#include <sys/sysmacros.h>	/* major() & minor() on SVR4 */
#endif

/* These are enum's on Linux, instead of #define's */
#ifdef Linux
#define AFORK	AFORK
#define ASU	ASU
#define ACORE	ACORE
#define AXSIG	AXSIG
#endif

/* options */
extern int numeric;
extern int verbose;

/* month strings */
static char *months[] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

void acct_header(void)
{
    
    printf("%-16s ", "command");    /* ACCT_COMM = 16 */
#if 0    
    printf("%4s ", "utim");
    printf("%4s ", "stim");
    printf("%4s ", "etim");
#endif
    if (numeric) {
	printf("%-9s ", "uid");
    }
    else {
	printf("%-9s ", "username");
    }
    if (numeric) {
	printf("%-9s ", "gid");
    }
    else {
	printf("%-9s ", "group");
    }
    printf("%5s ", "dev ");
    printf("%4s ", "flag");
#if 0    
if (verbose) {    
    printf("%6s ", "minflt");
    printf("%6s ", "majflt");
}
    printf("%2s ", "e");
#endif
    printf("time\n");
    
    return;
}

void acct_dump(const unsigned char *buf)
{
    struct acct *entry = (struct acct *) buf;
    struct passwd *pw;
    struct group *gr;
    struct tm *tp;
    
#ifdef DEBUG
    if (! entry) {
	fprintf(stderr, "DEBUG: dump_entry(): We got no entry!\n");
	return;
    }
#endif
    
    printf("%-16.*s ", (int) sizeof(entry->ac_comm), entry->ac_comm);

#if 0    
    printf("%4ld ", entry->ac_utime);
    
    printf("%4ld ", entry->ac_stime);
    
    printf("%4ld ", entry->ac_etime);
#endif    

    if (numeric) {
	printf("%-9ld ", (long) entry->ac_uid);
    }
    else {
	pw = getpwuid(entry->ac_uid);
	if (pw && pw->pw_name) {
	    printf("%-9s ", pw->pw_name);
	}
	else {
	    /* HEY: You have an entry for this uid, an no passwd entry! */
	    printf("!! %-6ld ", (long) entry->ac_uid);
	}
    }
    
    if (numeric) {
	printf("%-9ld ", (long) entry->ac_gid);
    }
    else {
	gr = getgrgid(entry->ac_gid);
	if (gr && gr->gr_name) {
	    printf("%-9s ", gr->gr_name);
	}
	else {
	    /* HEY: You have an entry for this gid, an no group entry! */
	    printf("!! %-6ld ", (long) entry->ac_gid);
	}
    }    
    
    printf("%2ld/%-2ld ", 
	   (long) major(entry->ac_tty),
	   (long) minor(entry->ac_tty));

    if (numeric) {
	printf("%04lo ", (long) entry->ac_flag);
    }
    else {
#ifdef AXSIG	
	if (entry->ac_flag & AXSIG)
	    putchar('X');
	else
#endif	    
	    putchar('-');
#ifdef ACORE	
	if (entry->ac_flag & ACORE)
	    putchar('C');
	else
#endif	    
	    putchar('-');
#ifdef ASU	
	if (entry->ac_flag & ASU)
	    putchar('S');
	else
#endif	    
	    putchar('-');
#ifdef AFORK	
	if (entry->ac_flag & AFORK)
	    putchar('F');
	else
#endif	    
	    putchar('-');
	putchar(' ');
    }

#if 0    
    if (verbose) {
	printf("%6ld ", entry->ac_minflt);
	
	printf("%6ld ", entry->ac_majflt);
    }
    
    printf("%2ld ", entry->ac_exitcode);
#endif

    tp = localtime((time_t *) &(entry->ac_btime));
    printf("%.3s %02d %02d:%02d:%02d %d\n", months[tp->tm_mon], tp->tm_mday,
	   tp->tm_hour, tp->tm_min, tp->tm_sec, 1900 + tp->tm_year);
    
    return;
}
