/*
 * dump.c
 * 
 * (!c) 2002 by Ighighi
 * Venezuela
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

#include <utmp.h>
#ifdef HAVE_UTMPX_H
#define __USE_GNU       1 /* hack to get [UW]TMPX pathnames on GNU */
#include <utmpx.h>
#endif
#include <lastlog.h>
#include <sys/acct.h>

#ifdef HAVE_PATHS_H
#include <paths.h>
#endif

#ifndef UTMP
#if     defined(_PATH_UTMP)
#define UTMP    _PATH_UTMP
#elif   defined(UTMP_FILE)
#define UTMP    UTMP_FILE
#elif   defined(UTMP_FILENAME)
#define UTMP    UTMP_FILENAME
#else
#error  "Please #define UTMP in config.h"
#endif
#endif  /* UTMP */

#ifndef WTMP
#if     defined(_PATH_WTMP)
#define WTMP    _PATH_WTMP
#elif   defined(WTMP_FILE)
#define WTMP    WTMP_FILE
#elif   defined(WTMP_FILENAME)
#define WTMP    WTMP_FILENAME
#else
#error  "Please #define WTMP in config.h"
#endif
#endif  /* WTMP */

#ifdef HAVE_UTMPX_H

#ifndef UTMPX
#if     defined(_PATH_UTMPX)
#define UTMPX   _PATH_UTMPX
#elif   defined(UTMPX_FILE)
#define UTMPX   UTMPX_FILE
#elif   defined(UTMPX_FILENAME)
#define UTMPX   UTMPX_FILENAME
#else
#error  "Please #define UTMPX in config.h"
#endif
#endif  /* UTMPX */

#ifndef WTMPX
#if     defined(_PATH_WTMPX)
#define WTMPX   _PATH_WTMPX
#elif   defined(WTMPX_FILE)
#define WTMPX   WTMPX_FILE
#elif   defined(WTMPX_FILENAME)
#define WTMPX   WTMPX_FILENAME
#else
#error  "Please #define WTMPX in config.h"
#endif
#endif  /* WTMPX */

#endif

#ifndef LASTLOG
#if     defined(_PATH_LASTLOG)
#define LASTLOG _PATH_LASTLOG
#elif   defined(LASTLOG_FILE)
#define LASTLOG LASTLOG_FILE
#elif   defined(LASTLOG_FILENAME)
#define LASTLOG LASTLOG_FILENAME
#else
#error  "Please #define LASTLOG in config.h"
#endif
#endif  /* LASTLOG */

/* options */
extern int numeric;
extern int verbose;

void utmp_header(void);
void utmpx_header(void);
void lastlog_header(void);
void acct_header(void);

void utmp_dump(const unsigned char *);
void utmpx_dump(const unsigned char *);
void lastlog_dump(const unsigned char *);
void acct_dump(const unsigned char *);

struct {
    char *file;
    int entry_size;
    void (*log_header)(void);
    void (*log_dump)(const unsigned char *);
} handler[] = {
      {
	  UTMP,
	  sizeof(struct utmp),
	  utmp_header,
	  utmp_dump
      },
      {
	  WTMP,
	  sizeof(struct utmp),
	  utmp_header,
	  utmp_dump
      },
#ifdef HAVE_UTMPX_H
      {
	  UTMPX,
	  sizeof(struct utmpx),
	  utmpx_header,
	  utmpx_dump
      },
      {
	  WTMPX,
	  sizeof(struct utmpx),
	  utmpx_header,
	  utmpx_dump
      },
#endif
      {
	  LASTLOG,
	  sizeof(struct lastlog),
	  lastlog_header,
	  lastlog_dump
      },
      {
	  NULL,
	  sizeof(struct acct),
	  acct_header,
	  acct_dump
      }
};

int dump(int index, const char *file)
{
    unsigned char *buf;
    int len, n;
    int fd;

    if (! file) {
	file = handler[index].file;
    }
    
    if (! file) {
	fprintf(stderr, "dump(): We got no file!\n");
	return -1;
    }

    if (verbose) {
	printf("Dumping file %s\n", file);
    }
    
    len = handler[index].entry_size;
    buf = (unsigned char *) malloc(len);
    if (! buf) {
	perror("malloc()");
	return -1;
    }
    
    fd = open(file, O_RDONLY);
    if (fd < 0) {
	fprintf(stderr, "open(): ");
	perror(file);
	return -1;
    }
    
    handler[index].log_header();
    
    while ( (n = read(fd, buf, len)) == len) {
	handler[index].log_dump(buf);
    }
    if (n) {
	fprintf(stderr, "read(): ");
	if (n < 0)
	    perror(file);
	else
	    fprintf(stderr, "%s: Possibly corrupted file\n", file);
	return -1;
    }
    
    if (close(fd) < 0) {
	fprintf(stderr, "close(): ");
	perror(file);
	return -1;
    }
    
    free(buf);
    
    return 0;
}
