/*
 * lastlog_dump.c
 * 
 * (!c) 2002 by Ighighi
 * Venezuela
 */

#include "config.h"

#include <stdio.h>
#include <time.h>
#include <lastlog.h>
#include <pwd.h>

/* options */
extern int numeric;	/* numeric output */
extern int verbose;	/* verbose output */

/* month strings */
static char *months[] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

void lastlog_header(void)
{    
    if (numeric) {
        printf("%-9s ", "uid");
    }
    else {
        printf("%-9s ", "username");
    }
    printf("%-9s ", "line");
    printf("%-20s ", "host");
    printf("time\n");
    
    return;
}

void lastlog_dump(const unsigned char *buf)
{
    struct lastlog *entry = (struct lastlog *) buf;
    struct passwd *pw;
    struct tm *tp;
    
    static int uid = -1;

#ifdef DEBUG
    if (! entry) {
	fprintf(stderr, "DEBUG: dump_entry(): We got no entry!\n");
	return;
    }
#endif
    
    ++uid;
    
    /* Skip null lastlog entries */
    if (! (entry->ll_line[0] ||
	   entry->ll_host[0] ||
	   entry->ll_time    )) {
	return;
    }
    
    if (numeric) {
	printf("%-9ld ", (long) uid);
    }
    else {
        pw = getpwuid(uid);
	if (pw && pw->pw_name) {
	    printf("%-9s ", pw->pw_name);
	}
	else {
	    /* HEY: You have an entry for this uid, and no passwd entry! */
	    printf("!! %-6ld ", (long) uid);
	}
    }

    printf("%-9.*s ",  (int) sizeof(entry->ll_line), entry->ll_line);
    printf("%-20.*s ", (int) sizeof(entry->ll_host), entry->ll_host);    
    
    tp = localtime(&entry->ll_time);
    printf("%.3s %02d %02d:%02d:%02d %d\n", months[tp->tm_mon], tp->tm_mday,
	   tp->tm_hour, tp->tm_min, tp->tm_sec, 1900 + tp->tm_year);

    return;
}
