/*
 * main.c
 * 
 * (!c) 2002 by Ighighi
 * Venezuela
 */

#define VERSION "logdump 1.0 by Ighighi"

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define UTMPDUMP	"utmpdump"
#define WTMPDUMP	"wtmpdump"
#define UTMPXDUMP	"utmpxdump"
#define WTMPXDUMP	"wtmpxdump"
#define LASTLOGDUMP	"lastlogdump"
#define ACCTDUMP	"acctdump"

/*
 * Function prototypes
 */

int dump(int index, const char *file);

static char *my_basename(char *pathname);
static void exit_usage(int status);
static void parse_args(int argc, char *argv[]);

/*
 * Global variables
 */

/* for all modules... */
int numeric	= 0;	/* numeric output */
int verbose	= 0;	/* verbose output */

/* for this module... */
static char *progname	= NULL;	/* program name */
static char *file	= NULL;	/* file to dump */

enum {
    utmp,
    wtmp,
#ifdef HAVE_UTMPX_H    
    utmpx,
    wtmpx,
#endif
    lastlog,
    acct
};

/*
 * my_basename()
 */
static char *my_basename(char *pathname)
{
    char *s;
    
    if (! pathname)
	return NULL;
    
    s = strrchr(pathname, '/');
    
    if (s)
	return ++s;
    else
	return pathname;
}

/*
 * exit_usage()
 */
static void exit_usage(int status)
{
    FILE *fp;

    if (! progname)
	return;
    
    if (status)
	fp = stderr;
    else
	fp = stdout;

    fprintf(fp, "Usage: %s [options]\n", progname);
    fprintf(fp, "%s\n", VERSION);
    fprintf(fp, "  options:\n");
    fprintf(fp, "  -f file, --file file     dump specified file\n");
    fprintf(fp, "  -n,      --numeric       numeric output\n");
    fprintf(fp, "  -v,      --verbose       verbose output\n");    
    fprintf(fp, "  -h,      --help          print help\n");
    fprintf(fp, "  -V,      --version       print version\n");
    
    exit(status);
    /* NOTREACH */
}

/*
 * parse_args()
 */
void parse_args(int argc, char *argv[])
{
    int i;

    progname = my_basename(argv[0]);
    
    for (i = 1; i < argc; i++) {
	if (argv[i][0] == '-') {
	    switch (argv[i][1]) {
	    case 'n':
		numeric = 1;
		break;
	    case 'v':
		verbose = 1;
		break;
	    case 'f':
		if (! (file = argv[++i]))
		    exit_usage(1);
		break;
	    case 'h':
		exit_usage(0);
		break;
	    case 'V':
		printf("%s\n", VERSION);
		exit(0);
		break;
	    default:
		fprintf(stderr, "%s: invalid option: %s\n", progname, argv[i]);
		exit_usage(1);
	    }
	}
	else {
	    break;
	}
    }

    if (i != argc) {
	fprintf(stderr, "%s: too many arguments\n", progname);
	exit_usage(1);
    }

    return;
}

int main(int argc, char *argv[])
{
    int n;

    parse_args(argc, argv);

    if      (! strcmp(progname, UTMPDUMP)) {
	n = dump(utmp, file);
    }
    else if (! strcmp(progname, WTMPDUMP)) {
	n = dump(wtmp, file);	
    }
#ifdef HAVE_UTMPX_H    
    else if (! strcmp(progname, UTMPXDUMP)) {
	n = dump(utmpx, file);
    }
    else if (! strcmp(progname, WTMPXDUMP)) {
	n = dump(wtmpx, file);
    }
#endif
    else if (! strcmp(progname, LASTLOGDUMP)) {
	n = dump(lastlog, file);
    }
    else if (! strcmp(progname, ACCTDUMP)) {
	n = dump(acct, file);
    }
    else {
#ifdef HAVE_UTMPX_H
	n = dump(utmpx, file);
#else
	n = dump(utmp, file);
#endif
    }    
    
    if (n < 0) {
	exit(1);
    }

    exit(0);
}
