/*
 * utmp_dump.c
 * 
 * (!c) 2002 by Ighighi
 * Venezuela
 */

#include "config.h"

#include <stdio.h>
#include <time.h>
#include <utmp.h>

#if defined(HAVE_UT_USER) && !defined(ut_name)
#define ut_name ut_user
#define HAVE_UT_NAME	1
#endif

#if defined(HAVE_UT_TV) && !defined(ut_time)
#define ut_time ut_tv.tv_sec
#define HAVE_UT_TIME	1
#endif

#if defined(HAVE_UT_ADDR_V6) && !defined(ut_addr)
#define ut_addr ut_addr_v6[0]
#define HAVE_UT_ADDR	1
#endif

#ifdef HAVE_UT_ADDR
#include <netinet/in.h>		/* struct in_addr */
#include <arpa/inet.h>		/* inet_ntoa() */
#endif

/* options */
extern int numeric;
extern int verbose;

/* month strings */
static char *months[] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

#ifdef HAVE_UT_TYPE
static char *ut_types[] = {
    "none",	/* EMPTY / UT_UNKNOWN	*/
    "runl",	/* RUN_LVL	*/
    "boot",	/* BOOT_TIME	*/
    "ntime",	/* NEW_TIME	*/
    "otime",	/* OLD_TIME	*/
    "init",	/* INIT_PROCESS	*/
    "login",	/* LOGIN_PROCESS	*/
    "user",	/* USER_PROCESS	*/
    "dead",	/* DEAD_PROCESS	*/
    "acct"	/* ACCOUNTING	*/
};
#undef	MAX_UT_TYPE
#define MAX_UT_TYPE	9
#endif

#ifdef HAVE_UT_PID
#undef	PID_SIZE
#define PID_SIZE	128	/* long enough to hold a pid */
static char pid[PID_SIZE];
#endif

void utmp_header(void)
{

#ifdef HAVE_UT_TYPE
    printf("%-5s ", "type");
#endif
#ifdef HAVE_UT_ID
    printf("%-4s ", "id");
#endif
#ifdef HAVE_UT_PID
    printf("%-5s ", "pid");
#endif
    printf("%-8s ", "line");
    printf("%-8s ", "user");
#ifdef HAVE_UT_HOST
    printf("%-19s ", "host");
#endif
    printf("time\n");

    return;
}

void utmp_dump(const unsigned char *buf)
{
    struct utmp *entry = (struct utmp *) buf;
    struct tm *tp;
    
#ifdef DEBUG	
    if (! entry) {
	fprintf(stderr, "DEBUG: dump_entry(): We got no entry!\n");
	return;
    }
#endif	
        
#ifdef HAVE_UT_TYPE
    if (numeric) {
	printf("%-5ld ", (long) entry->ut_type);
    }
    else {
	printf("%-5.5s ",
	       ((entry->ut_type < 0) || (entry->ut_type > MAX_UT_TYPE)) ?
		"none" : ut_types[entry->ut_type]);
    }
#endif    

#ifdef HAVE_UT_ID
    printf("%-4.*s ", (int) sizeof(entry->ut_id), entry->ut_id);
#endif    

#ifdef HAVE_UT_PID
    sprintf(pid, "%2ld", (long) entry->ut_pid);
    printf("%-5s ", pid);
#endif
    
    printf("%-8.*s ", (int) sizeof(entry->ut_line), entry->ut_line);
    
    printf("%-8.*s ", (int) sizeof(entry->ut_name), entry->ut_name);

#ifdef HAVE_UT_HOST
    printf("%-19.*s ", (int) sizeof(entry->ut_host), entry->ut_host);
#endif    

    tp = localtime(&(entry->ut_time));
    printf("%.3s %02d %02d:%02d:%02d", months[tp->tm_mon], tp->tm_mday,
	   tp->tm_hour, tp->tm_min, tp->tm_sec);
    if (verbose) {
	printf(".%06ld",
#ifdef HAVE_UT_TV		   
	       entry->ut_tv.tv_usec);
#else
	       (long) 0);
#endif		   
    }
    printf(" %d", 1900 + tp->tm_year);

    /* TODO: ut_addr_v6 */    
#ifdef HAVE_UT_ADDR
    if (entry->ut_addr) {
	char *addr = inet_ntoa(* (struct in_addr *) &(entry->ut_addr));
	if (addr) {
	    printf(" addr: %s", addr);
	}
    }
#endif

#ifdef HAVE_UT_SESSION
    if (entry->ut_session) {
	printf(" sid: %ld", (long) entry->ut_session);
    }
#endif

#ifdef HAVE_UT_EXIT
    if (entry->ut_exit.e_termination || entry->ut_exit.e_exit) {
	printf(" exit: %ld/%ld",
	       (long) entry->ut_exit.e_termination,
	       (long) entry->ut_exit.e_exit);
    }
#endif
    
    putchar('\n');

    return;
}
