/*
 * utmpx_dump.c
 * 
 * (!c) 2002 by Ighighi
 * Venezuela
 */

#include "config.h"

#ifdef HAVE_UTMPX_H

#include <stdio.h>
#include <time.h>
#define __USE_GNU 1	/* hack to get e_termination & e_exit on GNU */
#include <utmpx.h>

#if defined(HAVE_UTX_ADDR_V6) && !defined(ut_addr)
#define ut_addr ut_addr_v6[0]
#undef	HAVE_UTX_ADDR
#define HAVE_UTX_ADDR	1
#endif

#ifdef HAVE_UTX_ADDR
#include <netinet/in.h>		/* struct in_addr */
#include <arpa/inet.h>		/* inet_ntoa() */
#endif

/* options */
extern int numeric;
extern int verbose;

/* month strings */
static char *months[] = {
    "Jan", "Feb", "Mar", "Apr", "May", "Jun",
    "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};

static char *ut_types[] = {
    "none",	/* EMPTY / UT_UNKNOWN	*/
    "runl",	/* RUN_LVL	*/
    "boot",	/* BOOT_TIME	*/
    "ntime",	/* NEW_TIME	*/
    "otime",	/* OLD_TIME	*/
    "init",	/* INIT_PROCESS	*/
    "login",	/* LOGIN_PROCESS	*/
    "user",	/* USER_PROCESS	*/
    "dead",	/* DEAD_PROCESS	*/
    "acct"	/* ACCOUNTING	*/
};
#undef	MAX_UT_TYPE
#define MAX_UT_TYPE	9

#undef	PID_SIZE
#define PID_SIZE	128	/* long enough to hold a pid */
static char pid[PID_SIZE];

void utmpx_header(void)
{

    printf("%-5s ", "type");
    printf("%-4s ", "id");
    printf("%-5s ", "pid");
    printf("%-8s ", "line");
    printf("%-8s ", "user");
    printf("%-19s ", "host");
    printf("time\n");

    return;
}

void utmpx_dump(const unsigned char *buf)
{
    struct utmpx *entry = (struct utmpx *) buf;
    struct tm *tp;

#ifdef DEBUG    
    if (! entry) {
	fprintf(stderr, "DEBUG: dump_entry(): We got no entry!\n");
	return;
    }
#endif

    if (numeric) {
	printf("%-5ld ", (long) entry->ut_type);
    }
    else {
	printf("%-5.5s ", 
	       ((entry->ut_type < 0) || (entry->ut_type > MAX_UT_TYPE)) ?
	       "none" : ut_types[entry->ut_type]);
    }

    printf("%-4.*s ", (int) sizeof(entry->ut_id), entry->ut_id);

    sprintf(pid, "%2ld", (long) entry->ut_pid);
    printf("%-5s ", pid);
    
    printf("%-8.*s ", (int) sizeof(entry->ut_line), entry->ut_line);
    
    printf("%-8.*s ", (int) sizeof(entry->ut_user), entry->ut_user);

    /* TODO: ut_syslen */    
    printf("%-19.*s ", (int) sizeof(entry->ut_host), entry->ut_host);
    
    tp = localtime(&(entry->ut_tv.tv_sec));
    printf("%.3s %02d %02d:%02d:%02d", months[tp->tm_mon], tp->tm_mday,
	   tp->tm_hour, tp->tm_min, tp->tm_sec);
    if (verbose) {
	printf(".%06ld", entry->ut_tv.tv_usec);
    }
    printf(" %d", 1900 + tp->tm_year);

    /* TODO: ut_addr_v6 */    
#ifdef HAVE_UTX_ADDR
    if (entry->ut_addr) {
	char *addr = inet_ntoa(* (struct in_addr *) &(entry->ut_addr));
	if (addr) {
	    printf(" addr: %s", addr);
	}
    }
#endif
    
    if (entry->ut_session) {
	printf(" sid: %ld", (long) entry->ut_session);
    }

    if (entry->ut_exit.e_termination || entry->ut_exit.e_exit) {
	printf(" exit: %ld/%ld",
	       (long) entry->ut_exit.e_termination,
	       (long) entry->ut_exit.e_exit);
    }
    
    putchar('\n');

    return;
}

#endif	/* HAVE_UTMPX_H */
